import * as Comlink from "comlink";
import { assert, signalInvalidType } from "@canvas-js/utils";
import { ModelAPI } from "./ModelAPI.js";
export class InnerModelDB {
    db;
    #models = {};
    log;
    constructor(db, config, log) {
        this.db = db;
        for (const model of Object.values(config.models)) {
            this.#models[model.name] = new ModelAPI(this.db, model);
        }
        this.log = log;
    }
    close() {
        this.db.close();
    }
    apply(effects) {
        this.db.transaction(() => {
            for (const effect of effects) {
                const model = this.#models[effect.model];
                assert(model !== undefined, `model ${effect.model} not found`);
                if (effect.operation === "set") {
                    this.#models[effect.model].set(effect.value);
                }
                else if (effect.operation === "delete") {
                    this.#models[effect.model].delete(effect.key);
                }
                else {
                    signalInvalidType(effect);
                }
            }
        });
    }
    get(modelName, key) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return api.get(key);
    }
    getMany(modelName, keys) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return api.getMany(keys);
    }
    iterate(modelName, query = {}) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return Comlink.proxy(api.iterate(query));
    }
    count(modelName, where) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return api.count(where);
    }
    query(modelName, query = {}) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return api.query(query);
    }
    clear(modelName) {
        const api = this.#models[modelName];
        assert(api !== undefined, `model ${modelName} not found`);
        return api.clear();
    }
}
