export class Query {
    sql;
    statement;
    constructor(db, sql) {
        this.sql = sql;
        this.statement = db.prepare(sql);
    }
    get(params) {
        const statement = this.statement;
        const paramsWithColons = Object.fromEntries(Object.entries(params).map(([key, value]) => [":" + key, value]));
        try {
            if (Object.keys(paramsWithColons).length > 0)
                statement.bind(paramsWithColons);
            if (!statement.step()) {
                return null;
            }
            return statement.get({});
        }
        finally {
            statement.reset(true);
        }
    }
    all(params) {
        const statement = this.statement;
        const paramsWithColons = Object.fromEntries(Object.entries(params).map(([key, value]) => [":" + key, value]));
        try {
            if (Object.keys(paramsWithColons).length > 0)
                statement.bind(paramsWithColons);
            const result = [];
            while (statement.step()) {
                result.push(statement.get({}));
            }
            return result;
        }
        finally {
            statement.reset(true);
        }
    }
    *iterate(params) {
        const statement = this.statement;
        const paramsWithColons = Object.fromEntries(Object.entries(params).map(([key, value]) => [":" + key, value]));
        if (Object.keys(paramsWithColons).length > 0)
            statement.bind(paramsWithColons);
        const iter = {
            [Symbol.iterator]() {
                return this;
            },
            next() {
                const done = statement.step();
                if (done) {
                    return { done: true, value: undefined };
                }
                else {
                    return { done: false, value: statement.get({}) };
                }
            },
        };
        try {
            yield* iter;
        }
        finally {
            statement.reset(true);
        }
    }
}
export class Method {
    sql;
    statement;
    constructor(db, sql) {
        this.sql = sql;
        this.statement = db.prepare(sql);
    }
    run(params) {
        const statement = this.statement;
        const paramsWithColons = Object.fromEntries(Object.entries(params).map(([key, value]) => [":" + key, value]));
        if (Object.keys(paramsWithColons).length > 0)
            statement.bind(paramsWithColons);
        statement.step();
        statement.reset(true);
    }
}
